@@ -111,28 +111,40 @@ class Bootstrap(base.PollableMixin, base.GeneratedIdentifierMixin, base.Queryabl
         return self
 
 
+class Action(base.QueryableModel):
+    path = 'actions'
+    data_key = 'Actions'
+    primary_key = 'action_name'
+    fields = ('action_name', 'action_type', 'default_timeout', 'description', 'inputs',
+              'target_component', 'target_service', 'target_type')
+
+
 class Task(base.QueryableModel):
     path = 'tasks'
     data_key = 'Tasks'
     primary_key = 'id'
     fields = ('id', 'cluster_name', 'host_name', 'request_id', 'exit_code', 'stdout',
               'stderr', 'status', 'attempt_cnt', 'command', 'role', 'start_time',
-              'stage_id')
+              'stage_id', 'end_time', 'error_log', 'output_log', 'command_detail',
+              'structured_out', 'custom_command_name')
 
 
 class Request(base.PollableMixin, base.GeneratedIdentifierMixin, base.QueryableModel):
     path = 'requests'
     data_key = 'Requests'
     primary_key = 'id'
-    fields = ('id', 'request_context', 'status', 'progress_percent',
-              'queued_task_count', 'task_count', 'completed_task_count')
+    fields = ('id', 'request_context', 'status', 'request_status', 'progress_percent',
+              'queued_task_count', 'task_count', 'completed_task_count', 'type',
+              'operation_level', 'exclusive', 'aborted_task_count', 'create_time',
+              'end_time', 'failed_task_count', 'inputs', 'request_schedule',
+              'resource_filters', 'start_time', 'timed_out_task_count')
     relationships = {
         'tasks': Task,
     }
 
     @property
     def has_failed(self):
-        return True if self.status == 'FAILED' else False
+        return True if self.request_status == 'FAILED' else False
 
     @property
     def is_finished(self):
@@ -160,56 +172,82 @@ class Component(base.QueryableModel):
         return { 'name': self.component_name }
 
 
+class ClusterServerComponent(Component):
+    fields = ('cluster_name', 'component_name', 'service_name', 'category',
+              'installed_count', 'started_count', 'total_count')
+
+
 class HostComponentCollection(base.QueryableModelCollection):
+    @property
+    def _server_components(self):
+        """Client components can't be stopped or started, so we need to filter them out"""
+        server_components = []
+        for component in self:
+            service = component.cluster.services(component.service_name)
+            if service.components(component.component_name).category != 'CLIENT':
+                server_components.append(component)
+
+        return server_components
+
     def install(self):
         """Install all of the components associated with this host."""
-        self.load(self.client.put(self.url, data={
-            "RequestInfo": {
-                "context" :"Install All Components",
-                "operation_level": {
-                    "level": "HOST",
-                    "cluster_name": self.parent.cluster_name,
-                    "host_name": self.parent.host_name,
+        components = [x.component_name for x in self if x.state in ('INIT', 'UNINSTALLED')]
+        if components:
+            self.load(self.client.put(self.url, data={
+                "RequestInfo": {
+                    "context" :"Install All Host Components",
+                    "operation_level": {
+                        "level": "HOST",
+                        "cluster_name": self.parent.cluster_name,
+                        "host_name": self.parent.host_name,
+                    },
+                    "query": "HostRoles/component_name.in({0})".format(','.join(components)),
                 },
-            },
-            "Body": {
-                "HostRoles": {"state": "INSTALLED"},
-            },
-        }))
+                "Body": {
+                    "HostRoles": {"state": "INSTALLED"},
+                },
+            }))
         return self
 
     def start(self):
         """Start all of the components associated with this host."""
-        self.load(self.client.put(self.url, data={
-            "RequestInfo": {
-                "context" :"Start All Components",
-                "operation_level": {
-                    "level": "HOST",
-                    "cluster_name": self.parent.cluster_name,
-                    "host_name": self.parent.host_name,
+        components = [x.component_name for x in self._server_components
+                                           if x.state in ('INSTALLED', 'STOPPED')]
+        if components:
+            self.load(self.client.put(self.url, data={
+                "RequestInfo": {
+                    "context" :"Start All Host Components",
+                    "operation_level": {
+                        "level": "HOST",
+                        "cluster_name": self.parent.cluster_name,
+                        "host_name": self.parent.host_name,
+                    },
+                    "query": "HostRoles/component_name.in({0})".format(','.join(components)),
                 },
-            },
-            "Body": {
-                "HostRoles": {"state": "STARTED"},
-            },
-        }))
+                "Body": {
+                    "HostRoles": {"state": "STARTED"},
+                },
+            }))
         return self
 
     def stop(self):
         """Stop all of the components associated with this host."""
-        self.load(self.client.put(self.url, data={
-            "RequestInfo": {
-                "context" :"Stop All Components",
-                "operation_level": {
-                    "level": "HOST",
-                    "cluster_name": self.parent.cluster_name,
-                    "host_name": self.parent.host_name,
+        components = [x.component_name for x in self._server_components if x.state == 'STARTED']
+        if components:
+            self.load(self.client.put(self.url, data={
+                "RequestInfo": {
+                    "context" :"Stop All Host Components",
+                    "operation_level": {
+                        "level": "HOST",
+                        "cluster_name": self.parent.cluster_name,
+                        "host_name": self.parent.host_name,
+                    },
+                    "query": "HostRoles/component_name.in({0})".format(','.join(components)),
                 },
-            },
-            "Body": {
-                "HostRoles": {"state": "INSTALLED"},
-            },
-        }))
+                "Body": {
+                    "HostRoles": {"state": "INSTALLED"},
+                },
+            }))
         return self
 
 
@@ -219,7 +257,8 @@ class HostComponent(Component):
     data_key = 'HostRoles'
     fields = ('cluster_name', 'component_name', 'desired_stack_id',
               'desired_state', 'host_name', 'maintenance_state', 'service_name',
-              'stack_id', 'stale_configs', 'state', 'actual_configs')
+              'stack_id', 'stale_configs', 'state', 'actual_configs',
+              'desired_admin_state')
 
     def install(self):
         """Installs this component on the host in question."""
@@ -259,22 +298,23 @@ class HostComponent(Component):
 
     def restart(self):
         """Restarts this component on its host, if already installed and started."""
-        self.load(self.client.post(self.cluster.requests.url, data={
-            "RequestInfo": {
-                "command": "RESTART",
-                "context": "Restart %s" % normalize_underscore_case(self.component_name),
-                "operation_level": {
-                    "level": "SERVICE",
-                    "cluster_name": self.cluster_name,
-                    "service_name": self.service_name,
+        if self.state in ('STARTED', 'STOPPED', 'UNKNOWN'):
+            self.load(self.client.post(self.cluster.requests.url, data={
+                "RequestInfo": {
+                    "command": "RESTART",
+                    "context": "Restart %s" % normalize_underscore_case(self.component_name),
+                    "operation_level": {
+                        "level": "SERVICE",
+                        "cluster_name": self.cluster_name,
+                        "service_name": self.service_name,
+                    },
                 },
-            },
-            "Requests/resource_filters": [{
-                "service_name": self.service_name,
-                "component_name": self.component_name,
-                "hosts": self.host_name,
-            }],
-        }))
+                "Requests/resource_filters": [{
+                    "service_name": self.service_name,
+                    "component_name": self.component_name,
+                    "hosts": self.host_name,
+                }],
+            }))
         return self
 
 
@@ -379,7 +419,10 @@ class ClusterHost(Host):
 
     def load(self, response):
         if 'alerts' in response:
-            response['alerts'] = response['alerts']['detail']
+            if 'detail' in response['alerts']:
+                response['alerts'] = response['alerts']['detail']
+            else:
+                del response['alerts']
         return super(ClusterHost, self).load(response)
 
     def create(self, *args, **kwargs):
@@ -443,10 +486,14 @@ class Service(base.QueryableModel):
     data_key = 'ServiceInfo'
     primary_key = 'service_name'
     fields = ('service_name', 'cluster_name', 'maintenance_state', 'state')
+
+
+class ClusterService(Service):
     relationships = {
-        'components': Component,
+        'components': ClusterServerComponent,
     }
 
+
 class Configuration(base.QueryableModel):
     path = 'configurations'
     data_key = 'Config'
@@ -457,7 +504,6 @@ class Configuration(base.QueryableModel):
         # sigh, this API does not follow the pattern at all
         for field in self.fields:
             if field in response and field not in response['Config']:
-                LOG.error("Config has %s in main response body", field)
                 response['Config'][field] = response.pop(field)
         return super(Configuration, self).load(response)
 
@@ -501,6 +547,10 @@ class StackConfigurationList(Configuration):
 
         self.files = models
 
+    def to_dict(self):
+        self.inflate()
+        return { 'files': [x.to_dict() for x in self.files] }
+
 
 class Cluster(base.QueryableModel):
     path = 'clusters'
@@ -512,11 +562,36 @@ class Cluster(base.QueryableModel):
     relationships = {
         'hosts': ClusterHost,
         'requests': Request,
-        'services': Service,
+        'services': ClusterService,
         'configurations': Configuration,
-        'workflows': Workflow,
+# the workflows API doesn't appear to do anything yet
+#        'workflows': Workflow,
     }
 
+    def execute_action(self, action, context, parameters=None, hosts=None):
+        """Execute a custom action on the cluster.
+
+        The custom action gets executed on the specified hosts of the cluster.
+
+        :param action: Custom action name
+        :param context: Context name for the specific request
+        :param parameters: Dictionary of input parameters that are supported by
+                           the custom script
+        :param hosts: Comma separated list of hosts to run the action on
+        :return: Current status of the request
+        """
+        self.load(self.client.post(self.cluster.requests.url, data={
+            "RequestInfo": {
+                "action": action,
+                "context": context,
+                "parameters": parameters
+            },
+            "Requests/resource_filters": [{
+                "hosts": hosts,
+            }],
+        }))
+        return self.request
+
 
 class BlueprintHostGroup(base.DependentModel):
     fields = ('name', 'configurations', 'components')
@@ -564,7 +639,7 @@ class Repository(base.QueryableModel):
 
 
 class OperatingSystem(base.QueryableModel):
-    path = 'operatingSystems'
+    path = 'operating_systems'
     data_key = 'OperatingSystems'
     primary_key = 'os_type'
     fields = ('os_type', 'stack_name', 'stack_version')
@@ -607,7 +682,7 @@ class User(base.QueryableModel):
     path = 'users'
     data_key = 'Users'
     primary_key = 'user_name'
-    fields = ('user_name', 'active', 'admin', 'groups', 'ldap_user', 'password')
+    fields = ('user_name', 'active', 'admin', 'groups', 'ldap_user', 'password', 'old_password')
     relationships = {
         'privileges': UserPrivilege,
     }
